# CellTracking.py
# By MW, Jun 2013
# GPLv3+
#
# Class managing the States

import logging

class State :
    """Class representing a state
    A state is an intermediate object between a Movie and a player. It 
    contains many parameters that can be accessed by various modules"""
    def __init__(self, movie) :
        self.movie = movie
        self.subclasses_dict = {} 
        
        self.nb_frames = self.movie.get_frame_nb()
        self.size = self.movie.get_size()

    def add_subclass(self, name, inst) :
        if name in self.subclasses_dict.keys() :
            logging.error("Subclass name alredy registered. Please access it using get_subclass(name)")
        else :
            self.subclasses_dict[name] = inst
            logging.info("New registered subclass: %s", name)
            return inst

    def get_subclass(self, name) :
        try :
            return self.subclasses_dict[name]
        except :
            logging.error("Subclass '%s' not registered in this State", name)
    def has_subclass(self, name) :
        if name in self.subclasses_dict.keys() :
            return True
        else :
            return False
    def get_frame_nb(self) :
        return self.nb_frames
